@@ -1080,9 +1080,6 @@ libcrun_cgroup_enter (runtime_spec_schema_config_linux_resources *resources, int
 
   if (rootless > 0 && (cgroup_mode != CGROUP_MODE_UNIFIED || manager != CGROUP_MANAGER_SYSTEMD))
     {
-      if (cgroup_mode == CGROUP_MODE_UNIFIED && manager != CGROUP_MANAGER_SYSTEMD)
-        libcrun_warning ("cannot configure rootless cgroup using the cgroupfs manager");
-
       /* Ignore cgroups errors and set there is no cgroup path to use.  */
       free (*path);
       *path = NULL;
