@@ -18,7 +18,7 @@
 #define _GNU_SOURCE
 
 #include <config.h>
-#include <oci_runtime_spec.h>
+#include <runtime_spec_schema_config_schema.h>
 #include <stdbool.h>
 #include "container.h"
 #include "utils.h"
@@ -41,6 +41,7 @@
 #include <sys/epoll.h>
 #include <sys/socket.h>
 #include <grp.h>
+#include "sig2str.h"
 
 #ifdef HAVE_SYSTEMD
 # include <systemd/sd-daemon.h>
@@ -79,7 +80,7 @@ struct sync_socket_message_s
   char message[512];
 };
 
-typedef oci_container_hooks_prestart_element hook;
+typedef runtime_spec_schema_defs_hook hook;
 
 static char spec_file[] = "\
   {\n\
@@ -165,8 +166,8 @@ static char spec_file[] = "\
 				\"noexec\",\n\
 				\"newinstance\",\n\
 				\"ptmxmode=0666\",\n\
-				\"mode=0620\",\n\
-				\"gid=5\"\n\
+				\"mode=0620\"\
+%s\
 			]\n\
 		},\n\
 		{\n\
@@ -255,6 +256,9 @@ static char spec_file[] = "\
 	}\n\
 }\n";
 
+static const char *spec_pts_tty_group = ",\n\
+				\"gid=5\"\n";
+
 static const char *spec_user = "\
 			{\n\
 				\"type\": \"user\"\n \
@@ -379,7 +383,7 @@ sync_socket_send_sync (int fd, bool flush_errors, libcrun_error_t *err)
 }
 
 static libcrun_container_t *
-make_container (oci_container *container_def)
+make_container (runtime_spec_schema_config_schema *container_def)
 {
   libcrun_container_t *container = xmalloc (sizeof (*container));
   memset (container, 0, sizeof (*container));
@@ -394,9 +398,9 @@ make_container (oci_container *container_def)
 libcrun_container_t *
 libcrun_container_load_from_memory (const char *json, libcrun_error_t *err)
 {
-  oci_container *container_def;
+  runtime_spec_schema_config_schema *container_def;
   cleanup_free char *oci_error = NULL;
-  container_def = oci_container_parse_data (json, NULL, &oci_error);
+  container_def = runtime_spec_schema_config_schema_parse_data (json, NULL, &oci_error);
   if (container_def == NULL)
     {
       crun_make_error (err, 0, "load: %s", oci_error);
@@ -408,12 +412,12 @@ libcrun_container_load_from_memory (const char *json, libcrun_error_t *err)
 libcrun_container_t *
 libcrun_container_load_from_file (const char *path, libcrun_error_t *err)
 {
-  oci_container *container_def;
+  runtime_spec_schema_config_schema *container_def;
   cleanup_free char *oci_error = NULL;
-  container_def = oci_container_parse_file (path, NULL, &oci_error);
+  container_def = runtime_spec_schema_config_schema_parse_file (path, NULL, &oci_error);
   if (container_def == NULL)
     {
-      crun_make_error (err, 0, "load '%s': %s", path, oci_error);
+      crun_make_error (err, 0, "load `%s`: %s", path, oci_error);
       return NULL;
     }
   return make_container (container_def);
@@ -434,18 +438,31 @@ int block_signals (libcrun_error_t *err)
 static
 int unblock_signals (libcrun_error_t *err)
 {
+  int i;
   int ret;
   sigset_t mask;
+  struct sigaction act;
+
   sigfillset (&mask);
   ret = sigprocmask (SIG_UNBLOCK, &mask, NULL);
   if (UNLIKELY (ret < 0))
     return crun_make_error (err, errno, "sigprocmask");
+
+  memset (&act, 0, sizeof (act));
+  act.sa_handler = SIG_DFL;
+  for (i = 0; i < SIGNUM_BOUND; i++)
+    {
+      ret = sigaction (i, &act, NULL);
+      if (ret < 0 && errno != EINVAL)
+        return crun_make_error (err, errno, "sigaction");
+    }
+
   return 0;
 }
 
 /* must be used on the host before pivot_root(2).  */
 static int
-initialize_security (oci_container_process *proc, libcrun_error_t *err)
+initialize_security (runtime_spec_schema_config_schema_process *proc, libcrun_error_t *err)
 {
   int ret;
 
@@ -466,6 +483,73 @@ initialize_security (oci_container_process *proc, libcrun_error_t *err)
   return 0;
 }
 
+static int
+do_hooks (runtime_spec_schema_config_schema *def, pid_t pid, const char *id, bool keep_going, const char *rootfs,
+          const char *cwd, const char *status, hook **hooks, size_t hooks_len,
+          libcrun_error_t *err)
+{
+  size_t i, stdin_len;
+  int ret;
+  cleanup_free char *stdin = NULL;
+  const unsigned char *annotations = (const unsigned char *) "{}";
+  cleanup_free char *cwd_allocated = NULL;
+  yajl_gen gen = NULL;
+
+  if (cwd == NULL)
+    {
+      cwd = cwd_allocated = get_current_dir_name ();
+      if (cwd == NULL)
+        OOM ();
+    }
+
+  if (def && def->annotations && def->annotations->len)
+    {
+      size_t len;
+
+      gen = yajl_gen_alloc (NULL);
+      if (gen == NULL)
+        return crun_make_error (err, 0, "yajl_gen_alloc failed");
+
+      yajl_gen_map_open (gen);
+      for (i = 0; i < def->annotations->len; i++)
+        {
+          const char *key = def->annotations->keys[i];
+          const char *val = def->annotations->values[i];
+
+          yajl_gen_string (gen, YAJL_STR (key), strlen (key));
+          yajl_gen_string (gen, YAJL_STR (val), strlen (val));
+        }
+      yajl_gen_map_close (gen);
+
+      yajl_gen_get_buf (gen, &annotations, &len);
+    }
+
+  stdin_len = xasprintf (&stdin, "{\"ociVersion\":\"1.0\", \"id\":\"%s\", \"pid\":%i, \"root\":\"%s\", \"bundle\":\"%s\", \"status\":\"%s\", \"annotations\":%s}", id, pid, rootfs, cwd, status, annotations);
+
+  for (i = 0; i < hooks_len; i++)
+    {
+      ret = run_process_with_stdin_timeout_envp (hooks[i]->path, hooks[i]->args, cwd, hooks[i]->timeout, hooks[i]->env, stdin, stdin_len, err);
+      if (UNLIKELY (ret != 0))
+        {
+          if (keep_going)
+            libcrun_warning ("error executing hook `%s` (exit code: %d)", hooks[i]->path, ret);
+          else
+            {
+              libcrun_error (0, "error executing hook `%s` (exit code: %d)", hooks[i]->path, ret);
+              goto exit;
+            }
+        }
+    }
+
+  ret = 0;
+
+ exit:
+  if (gen)
+    yajl_gen_free (gen);
+
+  return ret;
+}
+
 /* Initialize the environment where the container process runs.
    It is used by the container init process.  */
 static int
@@ -479,8 +563,8 @@ container_init_setup (void *args, const char *notify_socket,
   int has_terminal;
   cleanup_close int console_socket = -1;
   cleanup_close int console_socketpair = -1;
-  oci_container *def = container->container_def;
-  oci_container_process_capabilities *capabilities;
+  runtime_spec_schema_config_schema *def = container->container_def;
+  runtime_spec_schema_config_schema_process_capabilities *capabilities;
   cleanup_free char *rootfs = NULL;
   int no_new_privs;
 
@@ -488,6 +572,10 @@ container_init_setup (void *args, const char *notify_socket,
   if (UNLIKELY (ret < 0))
     return ret;
 
+  ret = libcrun_configure_network (container, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
   rootfs = realpath (def->root->path, NULL);
   if (UNLIKELY (rootfs == NULL))
     {
@@ -525,6 +613,30 @@ container_init_setup (void *args, const char *notify_socket,
   if (UNLIKELY (ret < 0))
     return ret;
 
+  if (def->hooks && def->hooks->create_container_len)
+    {
+      ret = do_hooks (def, 0, container->context->id, false, def->root->path, NULL, "created",
+                      (hook **) def->hooks->create_container,
+                      def->hooks->create_container_len, err);
+      if (UNLIKELY (ret != 0))
+        return ret;
+    }
+
+  if (def->process)
+    {
+      ret = libcrun_set_selinux_exec_label (def->process, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+
+      ret = libcrun_set_apparmor_profile (def->process, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+    }
+
+  ret = close_fds_ge_than (entrypoint_args->context->preserve_fds + 3, err);
+  if (UNLIKELY (ret < 0))
+    crun_error_write_warning_and_release (entrypoint_args->context->output_handler_arg, &err);
+
   ret = libcrun_do_pivot_root (container, entrypoint_args->context->no_pivot, rootfs, err);
   if (UNLIKELY (ret < 0))
     return ret;
@@ -538,7 +650,7 @@ container_init_setup (void *args, const char *notify_socket,
 
       for (i = 0; i < def->process->env_len; i++)
         if (putenv (def->process->env[i]) < 0)
-          return crun_make_error (err, errno, "putenv '%s'", def->process->env[i]);
+          return crun_make_error (err, errno, "putenv `%s`", def->process->env[i]);
     }
 
   if (getenv ("HOME") == NULL)
@@ -595,17 +707,6 @@ container_init_setup (void *args, const char *notify_socket,
         }
     }
 
-  if (def->process)
-    {
-      ret = libcrun_set_selinux_exec_label (def->process, err);
-      if (UNLIKELY (ret < 0))
-        return ret;
-
-      ret = libcrun_set_apparmor_profile (def->process, err);
-      if (UNLIKELY (ret < 0))
-        return ret;
-    }
-
   ret = libcrun_set_hostname (container, err);
   if (UNLIKELY (ret < 0))
     return ret;
@@ -617,6 +718,9 @@ container_init_setup (void *args, const char *notify_socket,
         return ret;
     }
 
+  if (def->process->user)
+    umask (def->process->user->umask);
+
   if (def->process && !def->process->no_new_privileges)
     {
       char **seccomp_flags = NULL;
@@ -631,6 +735,8 @@ container_init_setup (void *args, const char *notify_socket,
       ret = libcrun_apply_seccomp (entrypoint_args->seccomp_fd, seccomp_flags, seccomp_flags_len, err);
       if (UNLIKELY (ret < 0))
         return ret;
+
+      close_and_reset (&entrypoint_args->seccomp_fd);
     }
 
   capabilities = def->process ? def->process->capabilities : NULL;
@@ -644,7 +750,7 @@ container_init_setup (void *args, const char *notify_socket,
       char *notify_socket_env;
       xasprintf (&notify_socket_env, "NOTIFY_SOCKET=%s", notify_socket);
       if (putenv (notify_socket_env) < 0)
-        return crun_make_error (err, errno, "putenv '%s'", notify_socket_env);
+        return crun_make_error (err, errno, "putenv `%s`", notify_socket_env);
     }
 
   return 0;
@@ -657,7 +763,7 @@ container_init (void *args, const char *notify_socket, int sync_socket,
 {
   struct container_entrypoint_s *entrypoint_args = args;
   int ret;
-  oci_container *def = entrypoint_args->container->container_def;
+  runtime_spec_schema_config_schema *def = entrypoint_args->container->container_def;
   cleanup_free const char *exec_path = NULL;
   entrypoint_args->sync_socket = sync_socket;
 
@@ -685,6 +791,8 @@ container_init (void *args, const char *notify_socket, int sync_socket,
   if (UNLIKELY (ret < 0))
     return ret;
 
+  close_and_reset (&sync_socket);
+
   if (entrypoint_args->context->fifo_exec_wait_fd >= 0)
     {
       char buffer[1];
@@ -705,14 +813,12 @@ container_init (void *args, const char *notify_socket, int sync_socket,
             return crun_make_error (err, errno, "read from the exec fifo");
         }
       while (ret == 0);
+
+      close_and_reset (&entrypoint_args->context->fifo_exec_wait_fd);
     }
 
   crun_set_output_handler (log_write_to_stderr, NULL, false);
 
-  ret = close_fds_ge_than (entrypoint_args->context->preserve_fds + 3, err);
-  if (UNLIKELY (ret < 0))
-    crun_error_write_warning_and_release (entrypoint_args->context->output_handler_arg, &err);
-
   if (def->process && def->process->no_new_privileges)
     {
       char **seccomp_flags = NULL;
@@ -727,6 +833,7 @@ container_init (void *args, const char *notify_socket, int sync_socket,
       ret = libcrun_apply_seccomp (entrypoint_args->seccomp_fd, seccomp_flags, seccomp_flags_len, err);
       if (UNLIKELY (ret < 0))
         return ret;
+      close_and_reset (&entrypoint_args->seccomp_fd);
     }
 
   if (UNLIKELY (def->process == NULL))
@@ -735,8 +842,16 @@ container_init (void *args, const char *notify_socket, int sync_socket,
   if (UNLIKELY (exec_path == NULL))
     return crun_make_error (err, errno, "executable path not specified");
 
-  if (def->process->user)
-    umask (def->process->user->umask);
+  if (def->hooks && def->hooks->start_container_len)
+    {
+      libcrun_container_t *container = entrypoint_args->container;
+
+      ret = do_hooks (def, 0, container->context->id, false, def->root->path, NULL, "starting",
+                      (hook **) def->hooks->start_container,
+                      def->hooks->start_container_len, err);
+      if (UNLIKELY (ret != 0))
+        return ret;
+    }
 
   execv (exec_path, def->process->args);
 
@@ -746,73 +861,6 @@ container_init (void *args, const char *notify_socket, int sync_socket,
   return crun_make_error (err, errno, "exec container process `%s`", exec_path);
 }
 
-static int
-do_hooks (oci_container *def, pid_t pid, const char *id, bool keep_going, const char *rootfs,
-          const char *cwd, const char *status, hook **hooks, size_t hooks_len,
-          libcrun_error_t *err)
-{
-  size_t i, stdin_len;
-  int ret;
-  cleanup_free char *stdin = NULL;
-  const unsigned char *annotations = (const unsigned char *) "{}";
-  cleanup_free char *cwd_allocated = NULL;
-  yajl_gen gen = NULL;
-
-  if (cwd == NULL)
-    {
-      cwd = cwd_allocated = get_current_dir_name ();
-      if (cwd == NULL)
-        OOM ();
-    }
-
-  if (def && def->annotations && def->annotations->len)
-    {
-      size_t len;
-
-      gen = yajl_gen_alloc (NULL);
-      if (gen == NULL)
-        return crun_make_error (err, 0, "yajl_gen_alloc failed");
-
-      yajl_gen_map_open (gen);
-      for (i = 0; i < def->annotations->len; i++)
-        {
-          const char *key = def->annotations->keys[i];
-          const char *val = def->annotations->values[i];
-
-          yajl_gen_string (gen, YAJL_STR (key), strlen (key));
-          yajl_gen_string (gen, YAJL_STR (val), strlen (val));
-        }
-      yajl_gen_map_close (gen);
-
-      yajl_gen_get_buf (gen, &annotations, &len);
-    }
-
-  stdin_len = xasprintf (&stdin, "{\"ociVersion\":\"1.0\", \"id\":\"%s\", \"pid\":%i, \"root\":\"%s\", \"bundle\":\"%s\", \"status\":\"%s\", \"annotations\":%s}", id, pid, rootfs, cwd, status, annotations);
-
-  for (i = 0; i < hooks_len; i++)
-    {
-      ret = run_process_with_stdin_timeout_envp (hooks[i]->path, hooks[i]->args, cwd, hooks[i]->timeout, hooks[i]->env, stdin, stdin_len, err);
-      if (UNLIKELY (ret != 0))
-        {
-          if (keep_going)
-            libcrun_warning ("error executing hook %s (exit code: %d)", hooks[i]->path, ret);
-          else
-            {
-              libcrun_error (0, "error executing hook %s (exit code: %d)", hooks[i]->path, ret);
-              goto exit;
-            }
-        }
-    }
-
-  ret = 0;
-
- exit:
-  if (gen)
-    yajl_gen_free (gen);
-
-  return ret;
-}
-
 static int
 read_container_config_from_state (libcrun_container_t **container, const char *state_root, const char *id, libcrun_error_t *err)
 {
@@ -823,18 +871,18 @@ read_container_config_from_state (libcrun_container_t **container, const char *s
 
   dir = libcrun_get_state_directory (state_root, id);
   if (UNLIKELY (dir == NULL))
-    return crun_make_error (err, 0, "cannot get state directory from %s", state_root);
+    return crun_make_error (err, 0, "cannot get state directory from `%s`", state_root);
 
   xasprintf (&config_file, "%s/config.json", dir);
   *container = libcrun_container_load_from_file (config_file, err);
   if (*container == NULL)
-    return crun_make_error (err, 0, "error loading %s", config_file);
+    return crun_make_error (err, 0, "error loading `%s`", config_file);
 
   return 0;
 }
 
 static int
-run_poststop_hooks (libcrun_context_t *context, oci_container *def,
+run_poststop_hooks (libcrun_context_t *context, runtime_spec_schema_config_schema *def,
                     libcrun_container_status_t *status,
                     const char *state_root, const char *id, libcrun_error_t *err)
 {
@@ -859,12 +907,12 @@ run_poststop_hooks (libcrun_context_t *context, oci_container *def,
         crun_error_write_warning_and_release (context->output_handler_arg, &err);
     }
   if (container && container->container_def)
-    free_oci_container (container->container_def);
+    free_runtime_spec_schema_config_schema (container->container_def);
   return 0;
 }
 
 static bool
-has_namespace_in_definition (oci_container *def, const char *namespace)
+has_namespace_in_definition (runtime_spec_schema_config_schema *def, const char *namespace)
 {
   size_t i;
 
@@ -880,7 +928,7 @@ has_namespace_in_definition (oci_container *def, const char *namespace)
 }
 
 static int
-container_delete_internal (libcrun_context_t *context, oci_container *def, const char *id, bool force, bool only_cleanup, libcrun_error_t *err)
+container_delete_internal (libcrun_context_t *context, runtime_spec_schema_config_schema *def, const char *id, bool force, bool only_cleanup, libcrun_error_t *err)
 {
   int ret;
   cleanup_container_status libcrun_container_status_t status;
@@ -910,7 +958,7 @@ container_delete_internal (libcrun_context_t *context, oci_container *def, const
           if (UNLIKELY (ret < 0))
             return ret;
           if (ret == 1)
-            return crun_make_error (err, 0, "the container '%s' is not in 'stopped' state", id);
+            return crun_make_error (err, 0, "the container `%s` is not in 'stopped' state", id);
         }
       else
         {
@@ -964,7 +1012,7 @@ container_delete_internal (libcrun_context_t *context, oci_container *def, const
 }
 
 int
-libcrun_container_delete (libcrun_context_t *context, oci_container *def, const char *id, bool force, libcrun_error_t *err)
+libcrun_container_delete (libcrun_context_t *context, runtime_spec_schema_config_schema *def, const char *id, bool force, libcrun_error_t *err)
 {
   return container_delete_internal (context, def, id, force, false, err);
 }
@@ -1189,7 +1237,7 @@ wait_for_process (pid_t pid, libcrun_context_t *context, int terminal_fd, int no
               if (UNLIKELY (ret < 0))
                 return ret;
               if (ret && context->detach)
-                    return 0;
+                return 0;
             }
           else if (events[i].data.fd == signalfd)
             {
@@ -1236,7 +1284,7 @@ flush_fd_to_err (libcrun_context_t *context, int terminal_fd)
 
   for (;;)
     {
-      int ret = read (terminal_fd, buf, sizeof (buf) - 1);
+      int ret = TEMP_FAILURE_RETRY (read (terminal_fd, buf, sizeof (buf) - 1));
       if (ret <= 0)
         break;
       buf[ret] = '\0';
@@ -1250,7 +1298,7 @@ flush_fd_to_err (libcrun_context_t *context, int terminal_fd)
 }
 
 static void
-cleanup_watch (libcrun_context_t *context, pid_t init_pid, oci_container *def, const char *id, int sync_socket, int terminal_fd)
+cleanup_watch (libcrun_context_t *context, pid_t init_pid, runtime_spec_schema_config_schema *def, const char *id, int sync_socket, int terminal_fd)
 {
   libcrun_error_t err = NULL;
   container_delete_internal (context, def, id, 1, true, &err);
@@ -1309,7 +1357,7 @@ open_seccomp_output (const char *id, int *fd, bool readonly, const char *state_r
 
 /* Find the uid:gid that is mapped to root inside the container user namespace.  */
 static void
-get_root_in_the_userns_for_cgroups (oci_container *def, uid_t host_uid, gid_t host_gid, uid_t *uid, gid_t *gid)
+get_root_in_the_userns_for_cgroups (runtime_spec_schema_config_schema *def, uid_t host_uid, gid_t host_gid, uid_t *uid, gid_t *gid)
 {
   *uid = -1;
   *gid = -1;
@@ -1349,7 +1397,7 @@ get_root_in_the_userns_for_cgroups (oci_container *def, uid_t host_uid, gid_t ho
 static int
 libcrun_container_run_internal (libcrun_container_t *container, libcrun_context_t *context, int container_ready_fd, libcrun_error_t *err)
 {
-  oci_container *def = container->container_def;
+  runtime_spec_schema_config_schema *def = container->container_def;
   int ret;
   pid_t pid;
   int detach = context->detach;
@@ -1514,6 +1562,17 @@ libcrun_container_run_internal (libcrun_container_t *container, libcrun_context_
           return ret;
         }
     }
+  if (def->hooks && def->hooks->create_runtime_len)
+    {
+      ret = do_hooks (def, pid, context->id, false, def->root->path, NULL, "created",
+                      (hook **) def->hooks->create_runtime,
+                      def->hooks->create_runtime_len, err);
+      if (UNLIKELY (ret != 0))
+        {
+          cleanup_watch (context, pid, def, context->id, sync_socket, terminal_fd);
+          return ret;
+        }
+    }
 
   ret = sync_socket_send_sync (sync_socket, true, err);
   if (UNLIKELY (ret < 0))
@@ -1583,7 +1642,7 @@ libcrun_container_run_internal (libcrun_container_t *container, libcrun_context_
 }
 
 static
-int check_config_file (oci_container *def, libcrun_error_t *err)
+int check_config_file (runtime_spec_schema_config_schema *def, libcrun_error_t *err)
 {
   if (UNLIKELY (def->root == NULL))
     return crun_make_error (err, 0, "invalid config file, no 'root' block specified");
@@ -1625,7 +1684,7 @@ int libcrun_copy_config_file (const char *id, const char *state_root, const char
 int
 libcrun_container_run (libcrun_context_t *context, libcrun_container_t *container, unsigned int options, libcrun_error_t *err)
 {
-  oci_container *def = container->container_def;
+  runtime_spec_schema_config_schema *def = container->container_def;
   int ret;
   int detach = context->detach;
   int container_ret_status[2];
@@ -1721,7 +1780,7 @@ libcrun_container_run (libcrun_context_t *context, libcrun_container_t *containe
 int
 libcrun_container_create (libcrun_context_t *context, libcrun_container_t *container, unsigned int options, libcrun_error_t *err)
 {
-  oci_container *def = container->container_def;
+  runtime_spec_schema_config_schema *def = container->container_def;
   int ret;
   int container_ready_pipe[2];
   cleanup_close int pipefd0 = -1;
@@ -1830,7 +1889,7 @@ libcrun_container_start (libcrun_context_t *context, const char *id, libcrun_err
     return ret;
 
   if (!ret)
-    return crun_make_error (err, errno, "container '%s' is not running", id);
+    return crun_make_error (err, errno, "container `%s` is not running", id);
 
   if (context->notify_socket)
     {
@@ -1994,7 +2053,7 @@ libcrun_container_state (libcrun_context_t *context, const char *id, FILE *out,
           }
         yajl_gen_map_close (gen);
       }
-   free_oci_container (container->container_def);
+   free_runtime_spec_schema_config_schema (container->container_def);
   }
 
   yajl_gen_map_close (gen);
@@ -2011,7 +2070,7 @@ libcrun_container_state (libcrun_context_t *context, const char *id, FILE *out,
 }
 
 int
-libcrun_container_exec (libcrun_context_t *context, const char *id, oci_container_process *process, libcrun_error_t *err)
+libcrun_container_exec (libcrun_context_t *context, const char *id, runtime_spec_schema_config_schema_process *process, libcrun_error_t *err)
 {
   int ret;
   pid_t pid;
@@ -2048,7 +2107,7 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
     return crun_make_error (err, 0, "error loading config.json");
 
   if (ret == 0)
-    return crun_make_error (err, 0, "the container '%s' is not running.", id);
+    return crun_make_error (err, 0, "the container `%s` is not running.", id);
 
   if (!context->detach)
     {
@@ -2091,11 +2150,11 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
       uid_t container_uid = process->user ? process->user->uid : 0;
       gid_t container_gid = process->user ? process->user->gid : 0;
       const char *cwd;
-      oci_container_process_capabilities *capabilities = NULL;
+      runtime_spec_schema_config_schema_process_capabilities *capabilities = NULL;
       char **seccomp_flags = NULL;
       size_t seccomp_flags_len = 0;
 
-      close (pipefd0);
+      TEMP_FAILURE_RETRY (close (pipefd0));
       pipefd0 = -1;
 
       cwd = process->cwd ? process->cwd : "/";
@@ -2108,7 +2167,7 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
 
       for (i = 0; i < process->env_len; i++)
         if (putenv (process->env[i]) < 0)
-          libcrun_fail_with_error ( errno, "putenv '%s'", process->env[i]);
+          libcrun_fail_with_error ( errno, "putenv `%s`", process->env[i]);
 
       if (getenv ("HOME") == NULL)
         {
@@ -2163,6 +2222,7 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
           ret = libcrun_apply_seccomp (seccomp_fd, seccomp_flags, seccomp_flags_len, err);
           if (UNLIKELY (ret < 0))
             return ret;
+          close_and_reset (&seccomp_fd);
         }
 
       if (process->user && process->user->additional_gids_len)
@@ -2195,13 +2255,14 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
           ret = libcrun_apply_seccomp (seccomp_fd, seccomp_flags, seccomp_flags_len, err);
           if (UNLIKELY (ret < 0))
             return ret;
+          close_and_reset (&seccomp_fd);
         }
 
       if (process->user)
         umask (process->user->umask);
 
       TEMP_FAILURE_RETRY (write (pipefd1, "0", 1));
-      close (pipefd1);
+      TEMP_FAILURE_RETRY (close (pipefd1));
       pipefd1 = -1;
 
       execv (exec_path, process->args);
@@ -2209,7 +2270,7 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
       _exit (EXIT_FAILURE);
     }
 
-  close (pipefd1);
+  TEMP_FAILURE_RETRY (close (pipefd1));
   pipefd1 = -1;
 
   if (seccomp_fd >= 0)
@@ -2252,7 +2313,7 @@ libcrun_container_exec (libcrun_context_t *context, const char *id, oci_containe
     }
 
   TEMP_FAILURE_RETRY (read (pipefd0, &b, sizeof (b)));
-  close (pipefd0);
+  TEMP_FAILURE_RETRY (close (pipefd0));
   pipefd0 = -1;
   if (b != '0')
     ret = -1;
@@ -2272,7 +2333,7 @@ libcrun_container_exec_process_file (libcrun_context_t *context, const char *id,
   struct parser_context ctx = {0, stderr};
   yajl_val tree = NULL;
   parser_error parser_err = NULL;
-  oci_container_process *process = NULL;
+  runtime_spec_schema_config_schema_process *process = NULL;
 
   ret = read_all_file (path, &content, &len, err);
   if (UNLIKELY (ret < 0))
@@ -2282,7 +2343,7 @@ libcrun_container_exec_process_file (libcrun_context_t *context, const char *id,
   if (UNLIKELY (ret < 0))
     return ret;
 
-  process = make_oci_container_process (tree, &ctx, &parser_err);
+  process = make_runtime_spec_schema_config_schema_process (tree, &ctx, &parser_err);
   if (UNLIKELY (process == NULL))
     {
       ret = crun_make_error (err, errno, "cannot parse process file");
@@ -2298,7 +2359,7 @@ libcrun_container_exec_process_file (libcrun_context_t *context, const char *id,
     yajl_tree_free (tree);
 
   if (process)
-    free_oci_container_process (process);
+    free_runtime_spec_schema_config_schema_process (process);
 
   return ret;
 }
@@ -2321,7 +2382,7 @@ libcrun_container_update (libcrun_context_t *context, const char *id, const char
 int
 libcrun_container_spec (bool root, FILE *out, libcrun_error_t *err arg_unused)
 {
-  return fprintf (out, spec_file, root ? "" : spec_user);
+  return fprintf (out, spec_file, root ? spec_pts_tty_group : "\n", root ? "" : spec_user);
 }
 
 int
@@ -2340,7 +2401,7 @@ libcrun_container_pause (libcrun_context_t *context, const char *id, libcrun_err
   if (UNLIKELY (ret < 0))
     return ret;
   if (ret == 0)
-    return crun_make_error (err, errno, "the container %s is not running", id);
+    return crun_make_error (err, errno, "the container `%s` is not running", id);
 
   return libcrun_container_pause_linux (&status, err);
 }
@@ -2361,7 +2422,7 @@ libcrun_container_unpause (libcrun_context_t *context, const char *id, libcrun_e
   if (UNLIKELY (ret < 0))
     return ret;
   if (ret == 0)
-    return crun_make_error (err, errno, "the container %s is not running", id);
+    return crun_make_error (err, errno, "the container `%s` is not running", id);
 
   return libcrun_container_unpause_linux (&status, err);
 }
