@@ -132,7 +132,11 @@ find_namespace (const char *name)
 static int
 syscall_clone (unsigned long flags, void *child_stack)
 {
+#if defined __s390__ || defined __CRIS__
+  return (int) syscall (__NR_clone, child_stack, flags);
+#else
   return (int) syscall (__NR_clone, flags, child_stack);
+#endif
 }
 
 static int
@@ -387,6 +391,8 @@ open_mount_target (libcrun_container_t *container, const char *target, libcrun_e
     return crun_make_error (err, 0, "target `%s` is not under the rootfs", target);
 
   target_rel = target + rootfslen + 1;
+  while (*target_rel == '/')
+    target_rel++;
 
   targetfd = openat (rootfsfd, target_rel, O_PATH | O_CLOEXEC);
   if (UNLIKELY (targetfd < 0))
@@ -736,17 +742,19 @@ struct device_s
   int major;
   int minor;
   int mode;
+  uid_t uid;
+  gid_t gid;
 };
 
 struct device_s needed_devs[] =
   {
-    {"/dev/null", "c", 1, 3, 0666},
-    {"/dev/zero", "c", 1, 5, 0666},
-    {"/dev/full", "c", 1, 7, 0666},
-    {"/dev/tty", "c", 5, 0, 0666},
-    {"/dev/random", "c", 1, 8, 0666},
-    {"/dev/urandom", "c", 1, 9, 0666},
-    {NULL, '\0', 0, 0, 0}
+   {"/dev/null", "c", 1, 3, 0666, 0, 0},
+   {"/dev/zero", "c", 1, 5, 0666, 0, 0},
+   {"/dev/full", "c", 1, 7, 0666, 0, 0},
+   {"/dev/tty", "c", 5, 0, 0666, 0, 0},
+   {"/dev/random", "c", 1, 8, 0666, 0, 0},
+   {"/dev/urandom", "c", 1, 9, 0666, 0, 0},
+   {NULL, '\0', 0, 0, 0, 0, 0}
   };
 
 /* Check if the specified path is a direct child of /dev.  If it is
@@ -832,6 +840,10 @@ create_dev (libcrun_container_t *container, int devfd, struct device_s *device,
           ret = chmod (fd_buffer, device->mode);
           if (UNLIKELY (ret < 0))
             return crun_make_error (err, errno, "fchmodat `%s`", device->path);
+
+          ret = chown (fd_buffer, device->uid, device->gid);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, errno, "chown `%s`", device->path);
         }
       else
         {
@@ -849,7 +861,7 @@ create_dev (libcrun_container_t *container, int devfd, struct device_s *device,
 
           if (ensure_parent_dir)
             {
-              ret = crun_ensure_directory (resolved_path, 0700, true, err);
+              ret = crun_ensure_directory (resolved_path, 0755, true, err);
               if (UNLIKELY (ret < 0))
                 return ret;
             }
@@ -883,6 +895,10 @@ create_dev (libcrun_container_t *container, int devfd, struct device_s *device,
           ret = chmod (fd_buffer, device->mode);
           if (UNLIKELY (ret < 0))
             return crun_make_error (err, errno, "fchmodat `%s`", device->path);
+
+          ret = chown (fd_buffer, device->uid, device->gid);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, errno, "chown `%s`", device->path);
         }
 
     }
@@ -925,7 +941,13 @@ create_missing_devs (libcrun_container_t *container, int rootfsfd, const char *r
                                 def->linux->devices[i]->type,
                                 def->linux->devices[i]->major,
                                 def->linux->devices[i]->minor,
-                                def->linux->devices[i]->file_mode};
+                                def->linux->devices[i]->file_mode,
+                                def->linux->devices[i]->uid,
+                                def->linux->devices[i]->gid,
+      };
+
+      if (! def->linux->devices[i]->file_mode_present)
+        device.mode = 0666;
       ret = create_dev (container, devfd, &device, rootfs, binds, true, err);
       if (UNLIKELY (ret < 0))
         return ret;
@@ -1154,6 +1176,8 @@ do_mounts (libcrun_container_t *container, int rootfsfd, const char *rootfs, lib
         }
 
       target_rel = target + rootfslen + 1;
+      while (*target_rel == '/')
+        target_rel++;
 
       type = def->mounts[i]->type;
 
